require('mootools');

var fs = require('fs');
var exec = require('child_process').exec;

var logger = require('node-simple-logger');

var async = require('async');
var redis = require("redis");
var pty = require('pty.js');

exports.createServer = function(options) {
	return new NodeRedis(options);
}

var NodeRedis = new Class({
	Implements: [Options, Events],
	options: {
		redis: {
			folder: 'redis',
			debug: false,
			forceStopOnStart: true,
			config: {
				port: 6379,
				bind: '127.0.0.1'
			}
		},
		web: {
			port: 6380
		}
	},
	
	initialize: function(options) {
		var self = this;
		
		self.running = false;
		
		// Setup options
		self.setOptions(options);
		
		// Check if there are already a redis started
		if(self.options.redis.forceStopOnStart) {
			exec('pidof -s redis-server', function(error, stdout, stderr) {
				// If yes, try to stop it properly with redis-cli
				if(stdout != "") {
					logger.info('Redis instance is already running, trying to stop it properly ...');
					exec('redis-cli -h ' + self.options.redis.config.bind + ' -p ' + self.options.redis.config.port + ' shutdown', function(error, stdout, stderr) {
						if(error) {
							logger.warn('Could not kill already started redis-server process. If you are running several redis server on this machine, please desactivate "forceStopOnStart" option. Try to continue with no');
						}
						else {
							logger.info('Other Redis instance is stopped !');
						}
						
						self.start();
					});
				}
				// If not, we can start our redis with not problem !
				else {
					self.start();
				}
			});
		}
		else {
			self.start();
		}
		
		// Stop redis on program exit
		process.on('SIGINT', function () { self.stop(); });
		process.on('exit', function () { self.stop(); });
	},
	
	on: function(eventName, callback) {
		var self = this;
		
		self.addEvent(eventName, callback);
	},
	
	start: function() {
		var self = this;
		
		async.series([
			function(callback) {
				self.initRedisConfig(callback);
			},
			function(callback) {
				self.startRedis(callback);
			},
			function(callback) {
				self.createRedisClient(callback);
			},
			function(callback) {
				self.startRedisWeb(callback);
			}
		],
		function(err) {
			if(err) logger.error(err);
			else {
				self.running = true;
				self.fireEvent('started');
			}
		});
	},
	
	stop: function() {
		var self = this;
		
		// Do not stop redis if it is not started
		if(self.running) {
			self.running = false;
			
			logger.info("Stopping Redis server ...");
			
			async.series([
				function(callback) {
					exec('redis-cli -h ' + self.options.redis.config.bind + ' -p ' + self.options.redis.config.port + ' shutdown', function() {
						callback(null);
					}, 500);
				}
			], function(err, results) {
				process.exit();
			});
		}
	},
	
	initRedisConfig: function(callback) {
		var self = this;
		
		// Create the redis folder if not exists
		if(!fs.existsSync(self.options.redis.folder)) {
			fs.mkdirSync(self.options.redis.folder);
		}
		
		// Generate content of the new configuration file
		var redisConfigFileContent = '# Redis configuration file\n# Autogenerated by node-redis-embed.\n';
		for(var attr in self.options.redis.config) {
			redisConfigFileContent += '\n' + attr + ' ' + self.options.redis.config[attr];
		}
		redisConfigFileContent += '\n';
		
		// Writing base configuration into file
		fs.writeFile(self.options.redis.folder + '/redis.conf', redisConfigFileContent, function (err) {
			if (err) callback(err);
		});
		
		callback(null);
	},
	
	startRedis: function(callback) {
		var self = this;
		
		var redisOk = false;
		
		logger.debug('Starting Redis Server ...');
		self.redisServer = pty.spawn('redis-server', ['redis.conf'], {
			cwd: 'redis',
			env: process.env
		});
		
		self.redisServer.on('data', function(data) {
			if(data.indexOf('The server is now ready to accept connections') > -1) {
				logger.debug('Redis Server is now ready to accept connections.');
				
				redisOk = true;
				callback(null);
			}
			
			// Debbuging redis
			if(self.options.redis.debug) {
				console.log(data);
			}
		});
		
		setTimeout(function() {
			if(redisOk === false) {
				callback('Could not start Redis !');
			}
		}, 2000);
	},
	
	createRedisClient: function(callback) {
		var self = this;
		
		self.client = redis.createClient(self.options.redis.config.port, self.options.redis.config.bind);
		self.client.select(0, function() {
			callback(null);
		});
		
		self.client.on('error', function (err) {
			callback('Redis client error ' + err);
		});
	},
	
	startRedisWeb: function(callback) {
		var self = this;
		
		logger.info('Redis web panel is listening at port : ' + self.options.web.port);
		self.redisCommander = pty.spawn('redis-commander', ['--redis-port', self.options.redis.config.port, '--redis-host', self.options.redis.config.bind, '--port', self.options.web.port, '--redis-db', 0], {
			cwd: 'redis',
			env: process.env
		});
		
		callback(null);
	},
});